import sys
import os

doc_dir = os.path.dirname(__file__)
mod_dir = os.path.dirname(doc_dir)

print('doc directory is: {}'.format(doc_dir))
print('module directory is: {}'.format(mod_dir))


sys.path.append(r'/Library/Python/2.7/site-packages/')
sys.path.append(mod_dir)


# -- General configuration ------------------------------------------------
# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.napoleon']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'rpws'
copyright = u'2017, eirannejad'
author = u'eirannejad'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = u'1'
# The full version, including alpha/beta/rc tags.
release = u'0'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'python'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.

# try to load the readthedocs module. otherwise use a standard theme
# this will cause the local sphinx to use a standard them (since readthedocs module is not installed),
# and the readthedocs.com bulder engine will use the readthedocs theme

try:
    import sphinx_rtd_theme
    html_theme = "sphinx_rtd_theme"
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
except:
    html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'rpwsdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'rpws.tex', u'rpws documentation',
     u'eirannejad', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'rpws', u'rpws documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'rpws', u'rpws documentation',
     author, 'rpws', 'python wrapper for Autodesk Revit Server.',
     ''),
]


# autodoc settings
# autodoc_member_order = 'bysource'



import sys
import os.path as op

rpws_folder = op.dirname(op.dirname(__file__))
sys.path.append(rpws_folder)
print('sys.path + {}'.format(rpws_folder))

from rpws import RevitServer

import testconfig as config


rs = RevitServer(config.test_server_name, config.test_server_version)


for parent, folders, files, models in rs.walk(config.test_folder):
    print(parent)
    for fd in folders:
        print('\t@d {}'.format(fd.path))
    for f in files:
        print('\t@f {}'.format(f.path))
    for m in models:
        print('\t@m {}'.format(m.path))



from __future__ import print_function

import sys
import os.path as op
import time

rpws_folder = op.dirname(op.dirname(__file__))
sys.path.append(rpws_folder)
print('sys.path + {}'.format(rpws_folder))

import rpws

import testconfig as config


rs = rpws.RevitServer(config.test_server_name, config.test_server_version)


print("\nTesting server info -------------------------------------------------")
print(rs, end='\n')
print(rs.path, end='\n')
print(rs._api_path(), end='\n')


sinfo = rs.getinfo()
print(sinfo.name, end='\n')
print(sinfo.version, end='\n')
print(sinfo.machine_name, end='\n')
print(sinfo.roles, end='\n')
print(sinfo.access_level_types, end='\n')
print(sinfo.max_path_length, end='\n')
print(sinfo.max_name_length, end='\n')
print(sinfo.servers, end='\n')

sdriveinfo = rs.getdriveinfo()
print(sdriveinfo.drive_space, end='\n')
print(sdriveinfo.drive_freespace, end='\n')


print("\nTesting server listfiles and getfileinfo --------------------------")
for m in rs.listfiles():
    print(m)


print("\nTesting server listfolders and getfolderinfo ------------------------")
for f in rs.listfolders():
    print(f.name)
    print(f.path)
    print(f.locks_inprogress)
    print(rs.getfolderinfo(f.path))


print("\nTesting server listmodels and getmodelinfo --------------------------")
for m in rs.listmodels(config.test_folder):
    print(m.name)
    print(m.path)
    print(m.locks_inprogress)
    print(rs.getmodelinfo(m.path))


print("\nTesting server getprojectinfo ---------------------------------------")
for m in rs.listmodels(config.test_folder):
    print(m.name)
    print(m.path)
    print(rs.getmodelinfo(m.path))
    for p in rs.getprojectinfo(m.path).parameters:
        print(p)

print("\nTesting server lock -------------------------------------------------")
rs.lock(config.test_file)
try:
    rs.lock(config.test_folder)
except rpws.ServerForbiddenError:
    print('Success')

print("\nTesting server unlock -----------------------------------------------")
rs.unlock(config.test_file)
rs.unlock(config.test_folder)

print("\nTesting server mkdir- -----------------------------------------------")
rs.mkdir(config.test_mkfolder)

print("\nTesting server rename -----------------------------------------------")
rs.rename(config.test_mkfolder, config.test_newfoldername)

print("\nTesting server delete- -----------------------------------------------")
rs.rmdir(config.test_renamedfolder)

print("\nTesting server copy -------------------------------------------------")
rs.copy(config.test_file, config.test_cpyfile, overwrite=True)

print("\nTesting server move -------------------------------------------------")
rs.move(config.test_cpyfile, config.test_mvfile, overwrite=True)

time.sleep(1)
rs.delete(config.test_mvfile)


print("\nTesting model history------------------------------------------------")
for h in rs.getmodelhistory(config.test_mhistory).items:
    print(h)


print("\nTesting descendent locks --------------------------------------------")
for h in rs.getdescendentlocks(config.test_folder).items:
    print(h)

for h in rs.deletedescendentlocks(config.test_folder):
    print(h)



# Revit server execeptions -----------------------------------------------------
class ServerVersionNotSupported(Exception):
    pass


class ServerConnectionError(Exception):
    pass


class ServerTimeoutError(Exception):
    pass


class UnhandledException(Exception):
    pass


# Revit Server API http status codes  ------------------------------------------
# 400
class ServerBadRequestError(Exception):
    pass


# 403
class ServerForbiddenError(Exception):
    pass


# 404
class ServerFileNotFound(Exception):
    pass


# 405
class ServerMethodNotAllowedError(Exception):
    pass


# 414
class ServerURITooLongError(Exception):
    pass


# 500
class ServerInternalError(Exception):
    pass


# 501
class ServerNotImplemented(Exception):
    pass


# 503
class ServerServiceUnavailableError(Exception):
    pass



# Edit and set server name and version
test_server_name = '<put server name here>'
test_server_version = '201X'

# Edit and direct to your test folders on your server
test_folder = r'\Tests'
test_mkfolder = r'\Tests\New Folder'
test_newfoldername = 'Renamed folder'
test_renamedfolder = r'\Tests\Renamed folder'

# Edit and direct to your test models on your server
test_file = r'\Tests\TestModel.rvt'
test_cpyfile = r'\Tests\TestModelCopy.rvt'
test_mvfile = r'\Tests\TestModelMove.rvt'
test_mhistory = r'\Tests\TestModel.rvt'



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['rpws']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.6']

setup(name='rpws',
      version='1.0.0',
      description='python wrapper for Autodesk Revit Server',
      author='Ehsan Iran-Nejad',
      author_email='eirannejad@gmail.com',
      url='https://github.com/eirannejad/rpws',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=2.7',
     )



from collections import namedtuple
import datetime

import enum


class DateEntry(datetime.datetime):
    @classmethod
    def fromrsdatestring(cls, date_string):
        seconds_since_epoch = int(date_string[6:-2])/1000
        return cls.utcfromtimestamp(seconds_since_epoch)


class ServerRole(enum.Enum):
    Host = 0
    Accelerator = 1
    Admin = 2


class LockState(enum.Enum):
    Unlocked = 0
    Locked = 1
    LockedParent = 2
    LockedChild = 3
    BeingUnlocked = 4
    BeingLocked = 5


class ParamType(enum.Enum):
    System = 'system'
    Custom = 'custom'
    Shared = 'shared'
    Unknown = 'unknown'


class ParamDataType(enum.Enum):
    Length = 'length'
    Number = 'number'
    Material = 'material'
    Text = 'text'
    MultilineText = "multiline text"
    YesNo = 'yes/no'
    Unknown = 'unknown'


ServerInfo = namedtuple('ServerInfo', ['name',
                                       'version',
                                       'machine_name',
                                       'roles',
                                       'access_level_types',
                                       'max_path_length',
                                       'max_name_length',
                                       'servers'])


ServerDriveInfo = namedtuple('ServerDriveInfo', ['drive_space',
                                                 'drive_freespace'])


EntryInfo = namedtuple('EntryInfo', ['path',
                                     'drive_space',
                                     'drive_freespace',
                                     'files',
                                     'folders',
                                     'lock_context',
                                     'lock_state',
                                     'locks_inprogress',
                                     'models'])


FileInfo = namedtuple('FileInfo', ['path', 'name', 'size', 'is_text'])


FolderInfo = namedtuple('FolderInfo', ['path', 'name', 'size',
                                       'has_contents',
                                       'lock_context',
                                       'lock_state',
                                       'locks_inprogress'])


ModelInfo = namedtuple('ModelInfo', ['path', 'name', 'size',
                                     'support_size',
                                     'product_version',
                                     'lock_context',
                                     'lock_state',
                                     'locks_inprogress'])


InProgressLockInfo = namedtuple('InProgressLockInfo',
                                ['age',
                                 'lock_options',
                                 'lock_type',
                                 'model_path',
                                 'timestamp',
                                 'username'])


DirectoryInfo = namedtuple('DirectoryInfo', ['path', 'name', 'size',
                                             'date_created',
                                             'date_modified',
                                             'exists',
                                             'folder_count',
                                             'is_folder',
                                             'last_modified_by',
                                             'lock_context',
                                             'lock_state',
                                             'model_count',
                                             'model_size',
                                             'locks_inprogress'])


ModelInfoEx = namedtuple('ModelInfoEx', ['path', 'name', 'size',
                                         'guid',
                                         'date_created',
                                         'date_modified',
                                         'last_modified_by',
                                         'support_size'])


ProjectInfo = namedtuple('ProjectInfo', ['parameters'])


ProjectParameter = namedtuple('ProjectParameter', ['name',
                                                   'value',
                                                   'id',
                                                   'category',
                                                   'type',
                                                   'datatype'])


ModelHistoryInfo = namedtuple('ModelHistoryInfo', ['path',
                                                   'items'])


ModelHistoryItemInfo = namedtuple('ModelHistoryItemInfo', ['id',
                                                           'comment',
                                                           'date',
                                                           'model_size',
                                                           'overwrittenby',
                                                           'support_size',
                                                           'user'])


ChildrenLockInfo = namedtuple('ChildrenLockInfo', ['path',
                                                   'items',
                                                   'lock_context'])
